@@ -1,6 +1,8 @@
-# coding: utf-8
 
-from typing import Any, Dict, Optional, List, Union, Optional, Iterator  # NOQA
+from __future__ import annotations
+
+if False:  # MYPY
+    from typing import Any, Dict, Optional, List, Union, Iterator  # NOQA
 
 anchor_attrib = '_yaml_anchor'
 
